package de.intevation.flys.artifacts.model.fixings;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;

import de.intevation.flys.artifacts.access.FixAnalysisAccess;

import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.FacetTypes;

import de.intevation.flys.artifacts.model.QWDDateRange;

import de.intevation.flys.artifacts.states.DefaultState.ComputeType;

import de.intevation.flys.utils.KMIndex;

import org.apache.log4j.Logger;


/**
 * Facet to show average W values for Q sectors.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixAvSectorFacet
extends      FixingsFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixAvSectorFacet.class);

    /** Trivial Constructor. */
    public FixAvSectorFacet() {
    }


    public FixAvSectorFacet(int ndx, String name, String description) {
        super(
            ndx,
            name,
            description,
            ComputeType.ADVANCE,
            null,
            null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact.
     * @param context  the CallContext.
     *
     * @return the data as QWD array (QWD[]).
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixAvSectorFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;
            FixAnalysisAccess access = new FixAnalysisAccess(flys, context);

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixAnalysisResult result = (FixAnalysisResult) res.getData();

            double currentKm = getCurrentKm(context);
            KMIndex<AnalysisPeriod []> kmPeriods = result.getAnalysisPeriods();
            KMIndex.Entry<AnalysisPeriod []> kmPeriodsEntry =
                kmPeriods.binarySearch(currentKm);

            if (kmPeriodsEntry == null) {
                logger.warn("No analysis periods found for km '" + currentKm + "'");
                return null;
            }

            AnalysisPeriod[] periods = kmPeriodsEntry.getValue();

            if (periods == null) {
                logger.warn("No analysis periods specified!");
                return null;
            }

            QWD[] qwdData = null;
            int sectorNdx = index & 3;
            int periodNdx = index >> 2;

            if (periodNdx < periods.length) {
                qwdData = periods[periodNdx].getQSectorAverages();
            }

            if (logger.isDebugEnabled()) {
                int resSize = qwdData != null ? qwdData.length : -1;
                logger.debug("Found " + resSize + " result elements.");
            }

            if (qwdData == null) {
                return null;
            }
            return new QWDDateRange(qwdData[sectorNdx], periods[periodNdx].getDateRange());
        }
        else {
            logger.warn("Artifact is no instance of FLYSArtifact.");
            return null;
        }
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixAvSectorFacet deepCopy() {
        FixAvSectorFacet copy = new FixAvSectorFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
